import dolfyn.adv.api as avm
from dolfyn.data.time import date2num, mpltime2matlab_datenum, num2date
import datetime
import numpy as np

"""
This data is from measurements at Admiralty Head, in admiralty
inlet. The measurements were made using an IMU equipped ADV mounted on
a mooring, the 'Tidal Turbulence Mooring' or 'TTM'. The inertial
measurements from the IMU allows for removal of mooring motion in post
processing. The mooring motion has been removed from the stream-wise
and vertical velocity signals (u, w). The lateral (v) velocity may
have some 'persistent motion contamination' due to mooring sway. The
ADV was positioned 11m above the seafloor in 58m of water at 48.1515N,
122.6858W.

Files
-----

This data contains the following files:

**<basefile>_<date>_Average5min.csv**

This file contains 5minute averages of the velocity, and the
turbulence intensity.

**<basename>_<date>.vec**

This is the full binary Nortek-format data file. For information on
this data format see Nortek's 'System Integration Manual' available
at: http://www.nortek-as.com/en/support/application-development

**<basename>_<date>_mc.h5**

This file contains a 'lightly processed' version of the raw data in
hdf5 format. This file can be read directly by the DOLfYN library's
'load' function, or using other hdf5 I/O tools such as HDFView.

**<basename>_<date>_mc.mat**

This file contains the key variables (u, urot, uacc, orientmat, Accel,
AngRt) from the .h5 file, in Matlab format.

**process_data.py**

A script that utilizes the DOLfYN library to read the binary .vec
file, and produce the other files in this dataset.


Units
-----

- Velocity data (_u, urot, uacc) is in m/s.
- Acceleration (Accel) data is in m/s^2.
- Angular rate (AngRt) data is in rad/s.
- The components of all vectors are in 'ENU' orientation. That is, the
  first index is True East, the second is True North, and the third is
  Up (vertical).
- All other quantities are in the units defined in the Nortek Manual.

Motion correction and rotation into the ENU earth reference frame was
performed using the Python-based open source DOLfYN library
(http://lkilcher.github.io/dolfyn/). Details on motion correction can
be found there.

For additional details on this dataset see the Thomson et al. 2013,
'Tidal Turbulence measurements from a compliant mooring' Marine Energy
Technology Symposium paper, available here:
http://faculty.washington.edu/jmt3rd/Publications/Thomson_etal_GMREC-METS_2013_turbulencemooring.pdf

"""


def within(dat, range):
    return (range[0] < dat) & (dat < range[1])

trange = [date2num(datetime.datetime(2012, 6, 12, 18, 15, 0)),
          date2num(datetime.datetime(2012, 6, 14, 14, 41, 0))]

filename = 'AdmiraltyInlet_June2012'

dat = avm.read_nortek(filename + '.vec')

dat.u[~within(dat.u, [-2.3, 0.5])] = np.NaN
avm.clean.fillpoly(dat.u, 3, 12)
dat.v[~within(dat.v, [-1, 1])] = np.NaN
avm.clean.fillpoly(dat.v, 3, 12)
dat.w[~within(dat.w, [-1.2, 0.2])] = np.NaN
avm.clean.fillpoly(dat.w, 3, 12)

dat.props['body2head_rotmat'] = np.eye(3)
# In meters, in ADV coord-system:
dat.props['body2head_vec'] = np.array([0.48, -0.07, -0.27])  # m
dat.props['lon'] = -122.6858
dat.props['lat'] = 48.1515
dat.props['height_above_bottom'] = 11  # m
dat.props['total_water_depth'] = 58  # m
dat.props['z'] = dat.props['height_above_bottom'] - dat.props['total_water_depth']
# Declination taken from http://www.ngdc.noaa.gov/geomag-web/#declination
#  Options: IGRF, 48 09.090', 122 41.149', 2012-06-12
dat.props['declination'] = 16.79

mc = avm.motion.CorrectMotion()
mc(dat)

# Crop the section of the data where the instrument was on the seafloor.
dat = dat.subset(within(dat.mpltime, trange))

dat.pitch, dat.roll, dat.heading = avm.rotate.orient2euler(dat.orientmat)

# Clean the data using the Goring+Nikora2002 method:
avm.clean.GN2002(dat.u)
avm.clean.GN2002(dat.v)
avm.clean.GN2002(dat.w)

# Now make the averaged data:
bnr = avm.TurbBinner(n_bin=5 * 60 * dat.fs, fs=dat.fs)
bdat = bnr(dat)

ti = bdat.sigma_Uh / bdat.U_mag
ti[bdat.U_mag < 0.7] = np.NaN

tmp = np.vstack((num2date(bdat.mpltime), bdat.u, bdat.v, bdat.w, ti)).T

np.savetxt(filename + '_Average5min.csv', tmp, fmt=['%s'] + ['%0.3f'] * 4,
           header='Date+Time (US/Pacific),'
           ' u (true east m/s), v (true north m/s), w (up m/s), Turbulence Intensity',
           delimiter=', ')

# Let's make the data float32 for file-size
for nm, d in dat:
    if isinstance(d, np.ndarray) and d.dtype == np.float64 and nm not in ['mpltime']:
        dat[nm] = d.astype(np.float32)

# Now save the data
dat.save(filename + '_raw.h5',
         units={'_u': 'm/s', 'urot': 'm/s', 'uacc': 'm/s', 'AngRt': 'rad/s',
                'Accel': 'm/s^2', 'AccelStable': 'm/s^2',
                'pitch': 'deg', 'roll': 'deg', 'heading': 'deg true',
                'mpltime': 'Python MatPlotLib Time format, Pacific Time', },
         description={'_u': 'Velocity array 0:True East, 1: True North, 2: Up',
                      'urot': 'The rotation-rate velocity.',
                      'uacc': 'The tranlational (acceleration) velocity.',
                      'AccelStable': 'The low-frequency acceleration that is ignored in calculating uacc.',
                      'pitch': 'The pitch of the ADV body',
                      'roll': 'The roll of the ADV body',
                      'heading': 'The heading (True) of the ADV body',
                      'orientmat': "The orientation matrix of the ADV body in the Earth's reference frame", })

# Now save the matlab file.
dat.add_data('datenum', mpltime2matlab_datenum(dat.mpltime), 'main')
dat.save_mat(filename + '_mc.mat', groups=['orient', 'main'])
dat.pop_data('datenum')
